clear all
format shortG 
warning('off','MATLAB:nearlySingularMatrix')

% add matlab functions to path
addpath('matlab_functions')

% load estimation file
load('data/full_formatted_data')

%% start simulation

% firm characteristics
property_value=100000;
employment=100;
obs=1;

% simulation characteristics
pref_loc=702;
possible_loc=struct_id(distances(pref_loc,:)<40);
ida_vector1=ida_vector;

% parameters 
beta_tax=-600;
ratio=624;
beta_distance=beta_tax/ratio;
kappa_prop=.29;
kappa_emp=43000;
beta_pov=0;

% simulation inputs
bidders_possibilities=[intersect(possible_loc,ida_vector1)];
bidders_possibilities=setdiff(bidders_possibilities,702);
counties=county_number(possible_loc);
unique_counties=unique(counties);
num_counties=length(unique_counties);

% storage
prob=zeros(6,1);
br=prob;

% see how amherst reacts when we introduce other IDAs

for idas_i = 1:6;
    
    % establish the set of bidders
    bidders=[702;bidders_possibilities]
    bidders=bidders(1:idas_i);
    bidders=sort(bidders);
    bidder_ind=ismember(possible_loc,bidders);
    num_idas=length(bidders);
    num_bidders=num_idas+num_counties;
    county_towns=cell(num_counties,1);
    county_ind=county_towns;
    for i=1:num_counties;
        county_towns{i}=possible_loc(counties==unique_counties(i));
        county_towns{i}=setdiff(county_towns{i},bidders);
        county_ind{i}=ismember(possible_loc,county_towns{i});
    end;
    valuations=repmat(kappa_prop*property_value+kappa_emp*employment,num_bidders,1);
    delta=beta_distance*distances(pref_loc,possible_loc)' + beta_tax*property_tax(possible_loc);

    % run the equilibrium
    ibr=ones(num_idas,1);
    new_ibr=ibr;
    cbr=ones(num_counties,1);
    new_cbr=cbr;
    dd=1;
    it=1;

    while (dd>0);

        % load choice variables from previous iteration
        ibr=new_ibr;
        cbr=new_cbr;
        delta_perm=zeros(1,length(delta));
        delta_county=county_towns;

        % compute deltas given the running bids
        % we'll call this one delta_perm and leave it constant
        % through the iteration.  we'll change delta as bidders
        % change their bid.

        for i=1:num_counties;

            delta_county{i}=beta_distance*distances(pref_loc,county_towns{i})' + ...
                beta_tax/15*((15-bid_options(cbr(i)))*ones(length(county_towns{i}),1)).*property_tax(county_towns{i});
            delta_perm(county_ind{i}==1)=delta_county{i};

        end;

        delta_ida=beta_distance*distances(pref_loc,bidders)' + beta_tax/15*((15*ones(num_idas,1)-bid_options(ibr)')).*property_tax(bidders);
        delta_perm(bidder_ind==1)=delta_ida;


        % now municipal idas
        for a=1:num_idas;

            delta=delta_perm;
            profits=zeros(10,1);

            for ida_br=1:10;

                delta(possible_loc==bidders(a))=beta_distance*distances(pref_loc,bidders(a))' + beta_tax/15*(15-bid_options(ida_br))*property_tax(bidders(a));
                profits(ida_br)=exp(delta(possible_loc==bidders(a)))/(sum(exp(delta)))...
                    *(valuations(length(unique_counties)+a)*(1+beta_pov*poverty_percent(bidders(a)))...
                    -bid_options(ida_br)*prop(obs)*property_tax(bidders(a)));
                inner_prob(ida_br)=exp(delta(possible_loc==bidders(a)))/(sum(exp(delta)))...
                    *(valuations(length(unique_counties)+a)*(1+beta_pov*poverty_percent(bidders(a)))...
                    -bid_options(ida_br)*prop(obs)*property_tax(bidders(a)));
            end;

            [~,new_ibr(a)]=max(profits);

        end;
        
           for ida_br=1:10;

                delta(possible_loc==pref_loc)=beta_distance*distances(pref_loc,pref_loc)' + beta_tax/15*(15-bid_options(ida_br))*property_tax(pref_loc);
                profits(ida_br)=exp(delta(possible_loc==pref_loc))/(sum(exp(delta)))...
                    *(valuations(length(unique_counties)+a)*(1+beta_pov*poverty_percent(bidders(a)))...
                    -bid_options(ida_br)*prop(obs)*property_tax(bidders(a)));
                inner_prob(ida_br)=exp(delta(possible_loc==pref_loc))/(sum(exp(delta)));
            end;


        dd=sum((new_cbr-cbr).^2) + sum((new_ibr-ibr).^2);

    end;
            
% pull the key stats from that iteration

prob(idas_i)=exp(delta_perm(possible_loc==pref_loc))/(sum(exp(delta_perm)));
br(idas_i)=bid_options(new_ibr(bidders==pref_loc))/15;


end;

% make plot 
hold on
[hAx,hLine1,hLine2] = plotyy(0:5,prob,0:5,br)
h_legend=legend( 'Win probability','Best-response','Location','SouthEast');
set(h_legend,'FontSize',14);
set(hAx(1),'YLim',[0 1])
set(hAx(2),'YLim',[0 1])
ylabel(hAx(1),'P(Amherst wins)','FontSize',16) 
ylabel(hAx(2),'Amherst bid (percent taxes exempt)','FontSize',16) 
xlabel(hAx(1),'Number of competing IDAs in firm choice set','FontSize',16) 
ax=gca;
ax.XTick = [0 1 2 3 4 5 6];
ax.YTick = [.2 .4 .6 .8 1];
set(gcf,'color','w');

hLine1.LineStyle = '--';
hLine2.LineStyle = '-';
hLine1.LineWidth = 3;
hLine2.LineWidth = 3;


[prob'; br']

