clear all
format shortG 
warning('off','MATLAB:nearlySingularMatrix')

% add matlab functions to path
addpath('matlab_functions')


% load estimation file
load('data/formatted_pov_data')


%% 1. compute covariance of moments to construct initial weighting matrix

moments=zeros(1000,34);

for bloop=1:1000;
    
    % pull a random sample of exemptions
    sample=datasample(1:num,num,'Replace',true);
    sample_exemptions_2013=property_exemptions_2013(sample);
    sample_exemptions_2008=property_exemptions_2008(sample);
    sample_ids=struct_id_auct(sample);
    sim_town_total_2008=struct_id;
    sim_town_total_2013=struct_id;
   
    for i=1:1096;
                
        sim_town_total_2013(i)=sum(sample_exemptions_2013(sample_ids==i));
        sim_town_total_2008(i)=sum(sample_exemptions_2008(sample_ids==i));     
        
    end;

    % the regression coefficients
    lprop_2013=log(sim_town_total_2013+1);
    lprop_2008=log(sim_town_total_2008+1);
    winsor_idas_within_25=idas_within_25;
    winsor_idas_within_25(idas_within_25>9)=9;
    winsor_idas_within_30=idas_within_30;
    winsor_idas_within_30(idas_within_30>12)=12;  
    winsor_idas_within_35=idas_within_35;
    winsor_idas_within_35(idas_within_35>18)=18;  
    winsor_idas_within_40=idas_within_40;
    winsor_idas_within_40(idas_within_40>20)=20;      
  
   z=[counties_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant,county_inds];      
    x=[idas_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant, county_inds];     
    z20=z;
    z20(:,1)=counties_within_20;
    x20=x;
    x20(:,1)=idas_within_20; 
    z30=z;
    z30(:,1)=counties_within_30;
    x30=x;
    x30(:,1)=idas_within_30;
    z35=z;
    z35(:,1)=counties_within_35;
    x35=x;
    x35(:,1)=idas_within_35;
    z40=z;
    z40(:,1)=counties_within_40;
    x40=x;
    x40(:,1)=idas_within_40;
    
    beta_20_true_2013=(z20'*x20)^-1*z20'*lprop_2013;    
    beta_25_true_2013=(z'*x)^-1*z'*lprop_2013;        
    beta_30_true_2013=(z30'*x30)^-1*z30'*lprop_2013;
    beta_35_true_2013=(z35'*x35)^-1*z35'*lprop_2013;
    beta_40_true_2013=(z40'*x40)^-1*z40'*lprop_2013;
    beta_20_true_2008=(z20'*x20)^-1*z20'*lprop_2008;        
    beta_25_true_2008=(z'*x)^-1*z'*lprop_2008;    
    beta_30_true_2008=(z30'*x30)^-1*z30'*lprop_2008;
    beta_35_true_2008=(z35'*x35)^-1*z35'*lprop_2008;
    beta_40_true_2008=(z40'*x40)^-1*z40'*lprop_2008;
    
    sim_b20_2013=beta_20_true_2013(1);
    sim_b20_2008=beta_20_true_2008(1);    
    sim_b25_2013=beta_25_true_2013(1);
    sim_b25_2008=beta_25_true_2008(1);     
    sim_b30_2013=beta_30_true_2013(1);
    sim_b30_2008=beta_30_true_2008(1);
    sim_b35_2013=beta_35_true_2013(1);
    sim_b35_2008=beta_35_true_2008(1); 
    sim_b40_2013=beta_40_true_2013(1);
    sim_b40_2008=beta_40_true_2008(1); 
    
    % get percent of towns giving subsidy;
    pos_ind_2013=0*sim_town_total_2013;
    pos_ind_2008=pos_ind_2013;
    pos_ind_2013(sim_town_total_2013>0)=1;
    pos_ind_2008(sim_town_total_2008>0)=1;
    percent_pos_2013=mean(pos_ind_2013);
    percent_pos_2008=mean(pos_ind_2008);
    
    % 25th percentile of subsidy
    p25_subs_2013=prctile(sample_exemptions_2013(sample_exemptions_2013>0),25);
    p25_subs_2008=prctile(sample_exemptions_2008(sample_exemptions_2008>0),25);
    
    % get sum of subsidy;
    sum_sim_2013=sum(sample_exemptions_2013);
    sum_sim_2008=sum(sample_exemptions_2008);
    
    % get valuation correlations
    emp_corr_2013=corr(log(emp(sample_exemptions_2013>0)+1),log(sample_exemptions_2013(sample_exemptions_2013>0)));
    emp_corr_2008=corr(log(emp(sample_exemptions_2008>0)+1),log(sample_exemptions_2008(sample_exemptions_2008>0)));

    % get poverty cdf
    
    mat_num_subs_2013=zeros(10,1);
    mat_num_subs_2008=mat_num_subs_2013;
    
    for i=1:1096;
        
        mat_num_subs_2013(i)=sum(sample_exemptions_2013(sample_ids==i).^0);
        mat_num_subs_2008(i)=sum(sample_exemptions_2008(sample_ids==i).^0); 
        
    end;
    
    mat_pov_cdf_2008=zeros(1,10);
    mat_pov_cdf_2013=mat_pov_cdf_2008;
    
    for k=1:10;
        
        mat_pov_cdf_2008(k)=sum(mat_num_subs_2013(poverty_percent<prctile(poverty_percent,10*k)))/sum(mat_num_subs_2013(:));
        mat_pov_cdf_2013(k)=sum(mat_num_subs_2013(poverty_percent<prctile(poverty_percent,10*k)))/sum(mat_num_subs_2013(:));            
   
    end;      

    % put in matrix
    moments(bloop,:)=[sim_b25_2013,sim_b30_2013,sim_b35_2013,sim_b40_2013,percent_pos_2013,p25_subs_2013,sum_sim_2013,emp_corr_2013, mat_pov_cdf_2013(1:9) ...
        sim_b25_2008,sim_b30_2008,sim_b35_2008,sim_b40_2008,percent_pos_2008,p25_subs_2008,sum_sim_2008,emp_corr_2008, mat_pov_cdf_2008(1:9)];

end;

moment_cov=cov(moments);
moment_cov=moment_cov.*eye(34);

%% 2. estimate auxiliary models to serve as target moments
    
    sample=1:num;
    sample_exemptions_2013=property_exemptions_2013(sample);
    sample_exemptions_2008=property_exemptions_2008(sample);
    sample_ids=struct_id_auct(sample);
    sim_town_total_2008=struct_id;
    sim_town_total_2013=struct_id;
   
    for i=1:1096;
                
        sim_town_total_2013(i)=sum(sample_exemptions_2013(sample_ids==i));
        sim_town_total_2008(i)=sum(sample_exemptions_2008(sample_ids==i));     
        
    end;

     lprop_2013=log(sim_town_total_2013+1);
     lprop_2008=log(sim_town_total_2008+1);

    % the regression coefficients
   
   z=[counties_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant,county_inds];      
    x=[idas_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant, county_inds];     
    z20=z;
    z20(:,1)=counties_within_20; 
    x20=x;
    x20(:,1)=idas_within_20;    
    z30=z;
    z30(:,1)=counties_within_30;
    x30=x;
    x30(:,1)=idas_within_30;
    z35=z;
    z35(:,1)=counties_within_35;
    x35=x;
    x35(:,1)=idas_within_35;
    z40=z;
    z40(:,1)=counties_within_40;
    x40=x;
    x40(:,1)=idas_within_40;
    
    beta_20_true_2013=(z20'*x20)^-1*z20'*lprop_2013;    
    beta_25_true_2013=(z'*x)^-1*z'*lprop_2013;        
    beta_30_true_2013=(z30'*x30)^-1*z30'*lprop_2013;
    beta_35_true_2013=(z35'*x35)^-1*z35'*lprop_2013;
    beta_40_true_2013=(z40'*x40)^-1*z40'*lprop_2013;
    beta_20_true_2008=(z20'*x20)^-1*z20'*lprop_2008;        
    beta_25_true_2008=(z'*x)^-1*z'*lprop_2008;    
    beta_30_true_2008=(z30'*x30)^-1*z30'*lprop_2008;
    beta_35_true_2008=(z35'*x35)^-1*z35'*lprop_2008;
    beta_40_true_2008=(z40'*x40)^-1*z40'*lprop_2008;
 
    true_b20_2013=beta_20_true_2013(1)   
    true_b20_2008=beta_20_true_2008(1)        
    true_b25_2013=beta_25_true_2013(1)
    true_b25_2008=beta_25_true_2008(1)       
    true_b30_2013=beta_30_true_2013(1)
    true_b30_2008=beta_30_true_2008(1) 
    true_b35_2013=beta_35_true_2013(1)
    true_b35_2008=beta_35_true_2008(1) 
    true_b40_2013=beta_40_true_2013(1)
    true_b40_2008=beta_40_true_2008(1) 
       
    % get percent of towns giving subsidy;
    pos_ind_2013=0*sim_town_total_2013;
    pos_ind_2008=pos_ind_2013;
    pos_ind_2013(sim_town_total_2013>0)=1;
    pos_ind_2008(sim_town_total_2008>0)=1;
    true_percent_pos_2013=mean(pos_ind_2013);
    true_percent_pos_2008=mean(pos_ind_2008);
    
    % 25th percentile of subsidy
    true_p25_subs_2013=prctile(sample_exemptions_2013(sample_exemptions_2013>0),25);
    true_p25_subs_2008=prctile(sample_exemptions_2008(sample_exemptions_2008>0),25);
    
    % get sum of subsidy;
    true_sum_2013=sum(sample_exemptions_2013);
    true_sum_2008=sum(sample_exemptions_2008);
    
    
    % get valuation correlations
    true_emp_corr_2013=corr(log(emp(active_2013==1 & property_exemptions_2013<1e6)+1),log(property_exemptions_2013(active_2013==1 & property_exemptions_2013<1e6)));
    true_emp_corr_2008=corr(log(emp(active_2008==1 & property_exemptions_2008<1e6)+1),log(property_exemptions_2008(active_2008==1 & property_exemptions_2008<1e6)));  

   % get poverty cdf
    
    num_subs_2013=zeros(10,1);
    num_subs_2008=num_subs_2013;
    
    for i=1:1096;
        
        num_subs_2013(i)=sum(sample_exemptions_2013(sample_ids==i).^0);
        num_subs_2008(i)=sum(sample_exemptions_2008(sample_ids==i).^0); 
        
    end;
    
    actual_pov_cdf_2008=zeros(1,10);
    actual_pov_cdf_2013=actual_pov_cdf_2008;
    
    for k=1:10;
        
        actual_pov_cdf_2008(k)=sum(num_subs_2013(poverty_percent<prctile(poverty_percent,10*k)))/sum(num_subs_2013(:));
        actual_pov_cdf_2013(k)=sum(num_subs_2013(poverty_percent<prctile(poverty_percent,10*k)))/sum(num_subs_2013(:));            
   
    end;      

%% 3. fix preferred locations to use across estimations 

% set the seed and initialize the matrix
rng('default') 
pref_loc_matrix=zeros(num,20);

for sim=1:50;
    
    for obs=1:num;
        
        % get the guys close enough to compete
        end_point=struct_id_auct(obs);
        possible_pref_loc=struct_id(distances(end_point,:)<40);
        sector=sector_auct(obs);
        year=exemption_year(obs);

        % get the starting point probability
        if (year<2012);
            
            mkt_share_vector=mkt_shares(:,(year-1993)*5+sector);
            
        end;
        
        if (year>2011);
            
            fake_year=2011;
            mkt_share_vector=mkt_shares(:,(fake_year-1993)*5+sector);
            
        end;
        
        input_shares=mkt_share_vector(possible_pref_loc);
        denominator=sum(input_shares);
        final_share=input_shares/denominator;
        
        if (denominator==0);
            
            final_share=input_shares.^0*(1/sum(input_shares.^0));
            
        end;
        
        % simulate a preferred location
        pref_loc_matrix(obs,sim) = datasample(possible_pref_loc,1,'Weights',final_share);
        
        
    end;
    
end;

%% 4. start parameter search by running a large coarse grid 
% note that the paper only includes one sigma calibration for the poverty
% estimation

sims=10;
factor_list=[300,600,900,1200,1500];
kappa_prop_list=[.1,.2,.3,.4,.5];
kappa_emp_list=[20000,40000,60000,80000];
beta_pov_list=[.1,.25,.5,1,2,3];
grid=allcomb(factor_list, kappa_prop_list, kappa_emp_list, beta_pov_list);
results_storage=zeros(length(grid1),5);

for grid_i=1:length(grid);
    param=grid(grid_i,:);
    results_storage(grid_i,1)=cal_pov_ii6(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                        bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                        true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                        true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                        true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                        true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
                                        actual_pov_cdf_2008, actual_pov_cdf_2013);
    results_storage(grid_i,2:5)=param;

end;

%% 5. continue with a finer grid

sims=10;
factor_list=[450,550,650,750,850];
kappa_prop_list=[.2,.23,.26,.29,.32];
kappa_emp_list=[10000,20000,40000,60000];
beta_pov_list=[.1,.25,.5,1,2,3];
grid=allcomb(factor_list, kappa_prop_list, kappa_emp_list, beta_pov_list);
results_storage=zeros(length(grid1),5);

for grid_i=1:length(grid);
    param=grid(grid_i,:);
    results_storage(grid_i,1)=cal_pov_ii6(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                        bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                        true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                        true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                        true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                        true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
                                        actual_pov_cdf_2008, actual_pov_cdf_2013);
    results_storage(grid_i,2:5)=param;

end;

%% 6. compute second step weighting matrix (using best parameter choice from finer grid)
sims=5;
pov_variance_results=zeros(sims*5,34);
param=[750, 0.29, 10000,.1];
simulated_moments=zeros(sims,34);

for big_i=1:5;
    
    start_point=sims*(big_i-1)+1;
    end_point=start_point+sims-1;
    results=pov_variance_function(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                        bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                        true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                        true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                        true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                        true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
                                        actual_pov_cdf_2008, actual_pov_cdf_2013);  

    pov_variance_results(start_point:end_point,:)=results
    
end;

trial_cov=cov(pov_variance_results);
sim_moment_cov=trial_cov.*0;
for i = 1:34;
    sim_moment_cov(i,i)=trial_cov(i,i);
end;

%% 7A. use simulated annealing algorithm from several starting points suggested by grid

sims=4;

% implicit function
f=@(param)anneal_pov_ii6(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                    bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                    true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                    true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                    true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
                                    actual_pov_cdf_2008, actual_pov_cdf_2013);

% bounds
l=[0,0,0,0];
u=[1,1,1,1];


% PS options
ps_options = psoptimset;
ps_options = psoptimset(ps_options,'MeshContraction', .75);
ps_options = psoptimset(ps_options,'MeshExpansion', 1.5);
ps_options = psoptimset(ps_options,'InitialMeshSize', .3);
ps_options = psoptimset(ps_options,'MaxFunEvals', 500);
ps_options = psoptimset(ps_options,'TolMesh', .01);
ps_options = psoptimset(ps_options,'TolX', .01);
ps_options = psoptimset(ps_options,'TolFun', .1);
ps_options = psoptimset(ps_options,'MeshRotate', 'on');
ps_options = psoptimset(ps_options,'Display', 'iter');

% SA options
options = saoptimset;
options = saoptimset(options,'TolFun', 1);
options = saoptimset(options,'StallIterLimit', 200);
options = saoptimset(options,'Display', 'iter');
options = saoptimset(options,'DisplayInterval', 1);
options = saoptimset(options,'HybridInterval', 'end');
options = saoptimset(options,'TemperatureFcn', @my_temperatureexp);
options = saoptimset(options,'TimeLimit', 86400);
options = saoptimset(options,'AcceptanceFcn', @my_acceptancesa);
options = saoptimset(options,'HybridInterval', 'end');
options = saoptimset(options,'HybridFcn', { @patternsearch, ps_options });
options = saoptimset(options,'InitialTemperature', [3,3,3]);
options = saoptimset(options,'ReannealInterval', 8);

% set starting points for parameters (which are rescaled in function)
start_points=[.33,.33,.33, .33; .5, .5, .5, .5; .66,.66,.66, .66];

% set storage for maximum arguments
sigma=zeros(3,4);


% call simulated annealing

for start_i= 1:3;
    
    start=start_points(start_i,:);
    
    [argmin1,fval1,exitflag1,output1] = ...
     simulannealbnd(f, start,l,u,options);
        
    sigma(start_i,:)=argmin1;

       
    
end;

%% 7B: reach final parameter estimates by running annealing algorithm from average argmax from 7A

start=mean(sigma,1);
[argmin1,fval1,exitflag1,output1] = ...
 simulannealbnd(f, start,l,u,options);
 
final_parameters=argmin1
 
% parameter estimates=
final_parameters








