clear all
format shortG 
warning('off','MATLAB:nearlySingularMatrix')

% add matlab functions to path
addpath('matlab_functions')

% load estimation file
load('data/formatted_services_data')

%% 1. compute covariance of moments to construct initial weighting matrix

moments=zeros(1000,16);

for cov_i=1:1000;
    
    % pull a random sample of exemptions
    sample=datasample(1:num,num,'Replace',true);
    sample_exemptions_2013=property_exemptions_2013(sample);
    sample_exemptions_2008=property_exemptions_2008(sample);
    sample_ids=struct_id_auct(sample);
    sim_town_total_2008=struct_id;
    sim_town_total_2013=struct_id;
   
    for i=1:1096;
                
        sim_town_total_2013(i)=sum(sample_exemptions_2013(sample_ids==i));
        sim_town_total_2008(i)=sum(sample_exemptions_2008(sample_ids==i));     
        
    end;

    % compute the regression coefficients
    lprop_2013=log(sim_town_total_2013+1);
    lprop_2008=log(sim_town_total_2008+1);  
  
   z=[counties_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant,county_inds];      
    x=[idas_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant, county_inds];     
    z20=z;
    z20(:,1)=counties_within_20;
    x20=x;
    x20(:,1)=idas_within_20; 
    z30=z;
    z30(:,1)=counties_within_30;
    x30=x;
    x30(:,1)=idas_within_30;
    z35=z;
    z35(:,1)=counties_within_35;
    x35=x;
    x35(:,1)=idas_within_35;
    z40=z;
    z40(:,1)=counties_within_40;
    x40=x;
    x40(:,1)=idas_within_40;
    
    beta_20_sim_2013=(z20'*x20)^-1*z20'*lprop_2013;    
    beta_25_sim_2013=(z'*x)^-1*z'*lprop_2013;        
    beta_30_sim_2013=(z30'*x30)^-1*z30'*lprop_2013;
    beta_35_sim_2013=(z35'*x35)^-1*z35'*lprop_2013;
    beta_40_sim_2013=(z40'*x40)^-1*z40'*lprop_2013;
    beta_20_sim_2008=(z20'*x20)^-1*z20'*lprop_2008;        
    beta_25_sim_2008=(z'*x)^-1*z'*lprop_2008;    
    beta_30_sim_2008=(z30'*x30)^-1*z30'*lprop_2008;
    beta_35_sim_2008=(z35'*x35)^-1*z35'*lprop_2008;
    beta_40_sim_2008=(z40'*x40)^-1*z40'*lprop_2008;
    
    sim_b20_2013=beta_20_sim_2013(1);
    sim_b20_2008=beta_20_sim_2008(1);    
    sim_b25_2013=beta_25_sim_2013(1);
    sim_b25_2008=beta_25_sim_2008(1);     
    sim_b30_2013=beta_30_sim_2013(1);
    sim_b30_2008=beta_30_sim_2008(1);
    sim_b35_2013=beta_35_sim_2013(1);
    sim_b35_2008=beta_35_sim_2008(1); 
    sim_b40_2013=beta_40_sim_2013(1);
    sim_b40_2008=beta_40_sim_2008(1); 
    
    % get percent of towns giving subsidy;
    pos_ind_2013=0*sim_town_total_2013;
    pos_ind_2008=pos_ind_2013;
    pos_ind_2013(sim_town_total_2013>0)=1;
    pos_ind_2008(sim_town_total_2008>0)=1;
    percent_pos_2013=mean(pos_ind_2013);
    percent_pos_2008=mean(pos_ind_2008);
    
    % 25th percentile of subsidy
    p25_subs_2013=prctile(sample_exemptions_2013(sample_exemptions_2013>0),25);
    p25_subs_2008=prctile(sample_exemptions_2008(sample_exemptions_2008>0),25);
    
    % get sum of subsidy;
    sum_sim_2013=sum(sample_exemptions_2013);
    sum_sim_2008=sum(sample_exemptions_2008);
    
    % get valuation correlations
    emp_corr_2013=corr(log(emp(sample_exemptions_2013>0)+1),log(sample_exemptions_2013(sample_exemptions_2013>0)));
    emp_corr_2008=corr(log(emp(sample_exemptions_2008>0)+1),log(sample_exemptions_2008(sample_exemptions_2008>0)));
  
    % put in matrix
    moments(cov_i,:)=[sim_b25_2013,sim_b30_2013,sim_b35_2013,sim_b40_2013,percent_pos_2013,p25_subs_2013,sum_sim_2013,emp_corr_2013, ...
        sim_b25_2008,sim_b30_2008,sim_b35_2008,sim_b40_2008,percent_pos_2008,p25_subs_2008,sum_sim_2008,emp_corr_2008];

end;

moment_cov=cov(moments);

%% 2. estimate auxiliary models to serve as target moments
    
    sample=1:num;
    sample_exemptions_2013=property_exemptions_2013(sample);
    sample_exemptions_2008=property_exemptions_2008(sample);
    sample_ids=struct_id_auct(sample);
    sim_town_total_2008=struct_id;
    sim_town_total_2013=struct_id;
   
    for i=1:1096;
                
        sim_town_total_2013(i)=sum(sample_exemptions_2013(sample_ids==i));
        sim_town_total_2008(i)=sum(sample_exemptions_2008(sample_ids==i));     
        
    end;

     lprop_2013=log(sim_town_total_2013+1);
     lprop_2008=log(sim_town_total_2008+1);

   % compute regression coefficients
   z=[counties_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant,county_inds];      
    x=[idas_within_25,pop, pop2, pop_within_25, pop2_within_25, cities_within_25, ...
        number_within_25, number2_within_25, border_within_25, estabs, estabs2, manu_percent, ...
        service_percent, business_percent, white_percent, poverty_percent, college_percent, tv_constant, county_inds];     
    z20=z;
    z20(:,1)=counties_within_20; 
    x20=x;
    x20(:,1)=idas_within_20;    
    z30=z;
    z30(:,1)=counties_within_30;
    x30=x;
    x30(:,1)=idas_within_30;
    z35=z;
    z35(:,1)=counties_within_35;
    x35=x;
    x35(:,1)=idas_within_35;
    z40=z;
    z40(:,1)=counties_within_40;
    x40=x;
    x40(:,1)=idas_within_40;
    
    beta_20_true_2013=(z20'*x20)^-1*z20'*lprop_2013;    
    beta_25_true_2013=(z'*x)^-1*z'*lprop_2013;        
    beta_30_true_2013=(z30'*x30)^-1*z30'*lprop_2013;
    beta_35_true_2013=(z35'*x35)^-1*z35'*lprop_2013;
    beta_40_true_2013=(z40'*x40)^-1*z40'*lprop_2013;
    beta_20_true_2008=(z20'*x20)^-1*z20'*lprop_2008;        
    beta_25_true_2008=(z'*x)^-1*z'*lprop_2008;    
    beta_30_true_2008=(z30'*x30)^-1*z30'*lprop_2008;
    beta_35_true_2008=(z35'*x35)^-1*z35'*lprop_2008;
    beta_40_true_2008=(z40'*x40)^-1*z40'*lprop_2008;
 
    true_b20_2013=beta_20_true_2013(1)   
    true_b20_2008=beta_20_true_2008(1)        
    true_b25_2013=beta_25_true_2013(1)
    true_b25_2008=beta_25_true_2008(1)       
    true_b30_2013=beta_30_true_2013(1)
    true_b30_2008=beta_30_true_2008(1) 
    true_b35_2013=beta_35_true_2013(1)
    true_b35_2008=beta_35_true_2008(1) 
    true_b40_2013=beta_40_true_2013(1)
    true_b40_2008=beta_40_true_2008(1) 
       
    % get percent of towns giving subsidy;
    pos_ind_2013=0*sim_town_total_2013;
    pos_ind_2008=pos_ind_2013;
    pos_ind_2013(sim_town_total_2013>0)=1;
    pos_ind_2008(sim_town_total_2008>0)=1;
    true_percent_pos_2013=mean(pos_ind_2013);
    true_percent_pos_2008=mean(pos_ind_2008);
    
    % 25th percentile of subsidy
    true_p25_subs_2013=prctile(sample_exemptions_2013(sample_exemptions_2013>0),25);
    true_p25_subs_2008=prctile(sample_exemptions_2008(sample_exemptions_2008>0),25);
    
    % get sum of subsidy;
    true_sum_2013=sum(sample_exemptions_2013);
    true_sum_2008=sum(sample_exemptions_2008);
    
    
    % get valuation correlations
    true_emp_corr_2013=corr(log(emp(active_2013==1 & property_exemptions_2013<1e6)+1),log(property_exemptions_2013(active_2013==1 & property_exemptions_2013<1e6)));
    true_emp_corr_2008=corr(log(emp(active_2008==1 & property_exemptions_2008<1e6)+1),log(property_exemptions_2008(active_2008==1 & property_exemptions_2008<1e6)));
  
%% 3. fix preferred locations to use across estimations 

% set the seed and initialize the matrix
rng('default') 
pref_loc_matrix=zeros(num,20);

for sim=1:50;
    
    for obs=1:num;
        
        % get the guys close enough to compete
        end_point=struct_id_auct(obs);
        possible_pref_loc=struct_id(distances(end_point,:)<40);
        sector=sector_auct(obs);
        year=exemption_year(obs);

        % get the starting point probability
        if (year<2012);
            
            mkt_share_vector=mkt_shares(:,(year-1993)*5+sector);
            
        end;
        
        if (year>2011);
            
            fake_year=2011;
            mkt_share_vector=mkt_shares(:,(fake_year-1993)*5+sector);
            
        end;
        
        input_shares=mkt_share_vector(possible_pref_loc);
        denominator=sum(input_shares);
        final_share=input_shares/denominator;
        
        if (denominator==0);
            
            final_share=input_shares.^0*(1/sum(input_shares.^0));
            
        end;
        
        % simulate a preferred location
        pref_loc_matrix(obs,sim) = datasample(possible_pref_loc,1,'Weights',final_share);
        
        
    end;
    
end;

%% 4. start parameter search by running a large coarse grid 

% set number of simulations per parameter draw (10 used in paper)
sims=10;

% set grid
beta_tax_list=[-200];
factor_list=[400, 600, 1000, 2000];
kappa_prop_list=[.2, .3, .4, .5];
kappa_emp_list=[10000,20000,40000,60000];
grid1=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-600];
factor_list=[400, 600, 1000, 2000];
kappa_prop_list=[.2, .3, .4, .5];
kappa_emp_list=[10000,20000,40000,60000];
grid2=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-1000];
factor_list=[400, 600, 1000, 2000];
kappa_prop_list=[.2, .3, .4, .5];
kappa_emp_list=[10000,20000,40000,60000];
grid3=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-2000];
factor_list=[400, 600, 1000, 2000];
kappa_prop_list=[.2, .3, .4, .5];
kappa_emp_list=[10000,20000,40000,60000];
grid4=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-3000];
factor_list=[400, 600, 1000, 2000];
kappa_prop_list=[.2, .3, .4, .5];
kappa_emp_list=[10000,20000,40000,60000];
grid5=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

grid=[grid1;grid2;grid3;grid4;grid5];
grid_dimensions=size(grid);
iterations=grid_dimensions(1)

distance_metric=zeros(iterations,1);
moments_storage=zeros(iterations,16);

% call function that evaluates distance metric at each parameter draw on
% the grid
grid_results=search_grid(struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, ...
    liability_2008, liability_2013, property_tax,bid_options, bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008, ...
    true_b40_2008,true_percent_pos_2008,true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008, ...
    active_2013, true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
    grid,distance_metric,moments_storage);   

%% 5. continue with a finer grid

sims=10;


beta_tax_list=[-400];
factor_list=[300,400,500,600,700,800,1000];
kappa_prop_list=[.2,.3,.4,.5,.6,.7,.8];
kappa_emp_list=[5000,15000,30000,60000,100000];
grid1=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-600];
factor_list=[300,400,500,600,700,800,1000];
kappa_prop_list=[.2,.3,.4,.5,.6,.7,.8];
kappa_emp_list=[5000,15000,30000,60000,100000];
grid2=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);

beta_tax_list=[-800];
factor_list=[300,400,500,600,700,800,1000];
kappa_prop_list=[.2,.3,.4,.5,.6,.7,.8];
kappa_emp_list=[5000,15000,30000,60000,100000];
grid3=allcomb(beta_tax_list, factor_list, kappa_prop_list, kappa_emp_list);


grid=[grid1,grid2,grid3];
grid_dimensions=size(grid);
iterations=grid_dimensions(2)

distance_metric=zeros(iterations,1);
moments_storage=zeros(iterations,16);

fine_grid_results=search_grid(struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, ...
    liability_2008, liability_2013, property_tax,bid_options, bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008, ...
    true_b40_2008,true_percent_pos_2008,true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008, ...
    active_2013, true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
    grid,distance_metric,moments_storage);   

%% 6. compute second step weighting matrix (using best parameter choice from finer grid)
sims=10;
param=[-600,800,.3,5000];

simulated_moments=zeros(5*sims,16);

variance_results=variance_intermediate(struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, ...
    liability_2008, liability_2013, property_tax,bid_options, bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008, ...
    true_b40_2008,true_percent_pos_2008,true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,active_2008, ...
    active_2013, true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013, ...
    simulated_moments,param);   

trial_cov=cov(variance_results);

% take the diagonal as weights
sim_moment_cov=trial_cov.*0;
for i = 1:16;
    sim_moment_cov(i,i)=trial_cov(i,i);
end;

%% 7A. use simulated annealing algorithm from several starting points suggested by grid

% set the number of simulations per parameter draw (4 used in paper)
sims=4;

% define objective function as implicit function to pass to SA algorithm
% different function for each sigma calibration

f4=@(param)cal_summer_ii4(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                    bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                    true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                    true_p25_subs_2008,true_sum_2008,num,sims, county_number,sim_moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                    true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013);

f6=@(param)cal_summer_ii6(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                    bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                    true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                    true_p25_subs_2008,true_sum_2008,num,sims, county_number,sim_moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                    true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013);

f8=@(param)cal_summer_ii8(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                    bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                    true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                    true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                    true_p25_subs_2008,true_sum_2008,num,sims, county_number,sim_moment_cov,pref_loc_matrix,active_2008,active_2013, ...
                                    true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013);
                                
% set parameter bounds
l=[0,0,0];
u=[1,1,1];

% options for simulated annealing algorithm
options = saoptimset;
options = saoptimset(options,'TolFun', 1);
options = saoptimset(options,'StallIterLimit', 200);
options = saoptimset(options,'Display', 'iter');
options = saoptimset(options,'DisplayInterval', 1);
options = saoptimset(options,'HybridInterval', 'end');
options = saoptimset(options,'TemperatureFcn', @my_temperatureexp);
options = saoptimset(options,'TimeLimit', 86400);
options = saoptimset(options,'AcceptanceFcn', @my_acceptancesa);
options = saoptimset(options,'HybridInterval', 'end');
options = saoptimset(options,'InitialTemperature', [3,3,3]);
options = saoptimset(options,'ReannealInterval', 8);

% options for pattern search
ps_options = psoptimset;
ps_options = psoptimset(ps_options,'MeshContraction', .75);
ps_options = psoptimset(ps_options,'MeshExpansion', 1.5);
ps_options = psoptimset(ps_options,'InitialMeshSize', .1);
ps_options = psoptimset(ps_options,'MaxFunEvals', 500);
ps_options = psoptimset(ps_options,'TolMesh', .01);
ps_options = psoptimset(ps_options,'TolX', .01);
ps_options = psoptimset(ps_options,'TolFun', .1);
ps_options = psoptimset(ps_options,'MeshRotate', 'on');
ps_options = psoptimset(options,'Display', 'iter');

% add a pattern search after convergence of the annealing algorithm
options = saoptimset(options,'HybridFcn', { @patternsearch, ps_options });

% set starting points for parameters (which are rescaled in function)
start_points=[.33,.33,.33; .5, .5, .5; .66,.66,.66];

% set storage for maximum arguments
sigma_25=zeros(3,3);
sigma_16=sigma_25;
sigma_13=sigma_16;


% call simulated annealing

for start_i= 1:3;
    
    start=start_points(start_i,:);
    
    [argmin1,fval1,exitflag1,output1] = ...
     simulannealbnd(f4, start,l,u,options);
        
    sigma_25(start_i,:)=argmin1;

    [argmin1,fval1,exitflag1,output1] = ...
     simulannealbnd(f6, start,l,u,options);
        
    sigma_16(start_i,:)=argmin1;

    [argmin1,fval1,exitflag1,output1] = ...
     simulannealbnd(f8, start,l,u,options);
        
    sigma_13(start_i,:)=argmin1;
       
    
end;

%% 7B: reach final parameter estimates by running annealing algorithm from average argmax from 7A

% for sigma = 2.5
start=mean(sigma_25,1);
[argmin1,fval1,exitflag1,output1] = ...
 simulannealbnd(f4, start,l,u,options);
 
final_parameters_sigma_25=argmin1
 
% for sigma = 1.6
start=mean(sigma_16,1);
[argmin1,fval1,exitflag1,output1] = ...
 simulannealbnd(f6, start,l,u,options);
 
final_parameters_sigma_16=argmin1

% for sigma = 1.3
start=mean(sigma_13,1);
[argmin1,fval1,exitflag1,output1] = ...
 simulannealbnd(f8, start,l,u,options);
 
final_parameters_sigma_13=argmin1

% parameter estimates for sigma=2.5=
final_parameters_sigma_25

% parameter estimates for sigma=1.6=
final_parameters_sigma_16

% parameter estimates for sigma=1.3=
final_parameters_sigma_13





