clear all
format shortG 
warning('off','MATLAB:nearlySingularMatrix')

% add matlab functions to path
addpath('matlab_functions')


%% compute standard errors for retail/services model

% load data specialized to services
clear
load('data/formatted_services_data')

% set number of simulations per parameter draw (10 used in paper)
sims=10;

% set relative step size for numerical derivative
up=1.05;

% set up a grid of that step size to evaluate derivative
box_grid=zeros(3,4,4);
box_grid(:,:,1)=[-400,-400/567,.275,48000; ...
 -600,-600/656,.211,38400; ...
 -800,-800/649,.209,59900];
box_grid(:,:,2)=box_grid(:,:,1);
box_grid(:,2,2)=box_grid(:,2,2)*up;
box_grid(:,:,3)=box_grid(:,:,1);
box_grid(:,3,3)=box_grid(:,3,3)*up;
box_grid(:,:,4)=box_grid(:,:,1);
box_grid(:,4,4)=box_grid(:,4,4)*up;

beta_dist=box_grid(:,2,1);
kappa_prop=box_grid(:,3,1);
kappa_emp=box_grid(:,4,1);

moments_storage_m=zeros(3,4,16);

% loop over tweaks to each parameter for each calibration of sigma
for cal_i=1:3;
    for param_i=1:4;

            param=box_grid(cal_i,:,param_i);
            boop=function_tweaks(param,struct_id,struct_id_auct,distances,poverty_percent,property_tax_2013,property_tax_2008,exemption_year,ida_vector,prop,emp, liability_2008, liability_2013, property_tax,bid_options, ...
                                bid_plans,z,x,z20,x20,z30,x30,z35,x35,z40,x40,true_b20_2013,true_b25_2013,true_b30_2013, ...
                                true_b35_2013,true_b40_2013,true_percent_pos_2013,true_p25_subs_2013,true_sum_2013, ...
                                true_b20_2008,true_b25_2008,true_b30_2008,true_b35_2008,true_b40_2008,true_percent_pos_2008,...
                                true_p25_subs_2008,true_sum_2008,num,sims, county_number,moment_cov,pref_loc_matrix,error_mat,active_2008,active_2013, ...
                                true_emp_corr_2013, true_emp_corr_2008,state_sales_exemptions_2013,property_exemptions_2008,property_exemptions_2013);
            moments_storage_m(cal_i,param_i,:)=boop;
            save('moments_storage_services.mat','moments_storage_m');

    end;
end;


% use those derivatives to compute standard errors
moments_storage=moments_storage_m;

deriv_dist=zeros(3,16);
deriv_prop=deriv_dist;
deriv_emp=deriv_dist;

for i = 1:3;
    
    deriv_dist(i,:)=squeeze((moments_storage(i,2,:)-moments_storage(i,1,:))/((up-1)*beta_dist(i)));
    deriv_prop(i,:)=squeeze((moments_storage(i,3,:)-moments_storage(i,1,:))/((up-1)*kappa_prop(i)));
    deriv_emp(i,:)=squeeze((moments_storage(i,4,:)-moments_storage(i,1,:))/((up-1)*kappa_emp(i)));
    
end;
    
std_errors=zeros(3,3);
for i = 1:3;
    
    g_matrix=[deriv_dist(i,:)',deriv_prop(i,:)',deriv_emp(i,:)'];
    param_cov=1.2*eye(3)/((g_matrix'/sim_moment_cov)*g_matrix);
    std_errors(i,:)=diag(param_cov).^.5

end;


% standard errors for retail/services sample
std_errors






